"""
Common logic to handle timestamps and dates
"""
from datetime import timedelta, datetime, timezone, tzinfo
from typing import Union, Any
from timeit import default_timer as timer

import pytz

DEFAULT_TZ: tzinfo = datetime.now(timezone.utc).astimezone().tzinfo


def to_utc(candidate: datetime) -> datetime:
    """
    Enable UTC for a given datetime
    :param candidate:
    :return:
    """
    try:
        converted = candidate.astimezone(pytz.utc)
    except (ValueError, TypeError):
        converted = candidate.replace(tzinfo=pytz.utc)
    return converted


DEFAULT_TIMESTAMP_10M_AGO: datetime = to_utc(datetime.now(tz=DEFAULT_TZ) - timedelta(minutes=10))
DEFAULT_TIMESTAMP_10Y_AGO: datetime = to_utc(datetime.now(tz=DEFAULT_TZ) - timedelta(days=365 * 10))


def parse_timestamp(candidate: Union[str, Any]) -> datetime:
    """
    Expected something like 2022-02-08T16:32:14.900292+0000
    :param candidate:
    :return:
    """
    if isinstance(candidate, str):
        try:
            iso_candidate = candidate.split('+', 1)[0]
            return to_utc(datetime.fromisoformat(iso_candidate))
        except ValueError as ex:
            raise ValueError(f"Invalid date passed: {candidate}") from ex
    else:
        return to_utc(candidate)


def get_clock(start_time: float) -> str:
    """
    Get the clock time from a timestamp, as pretty string
    :param start_time:
    :return:
    """
    seconds = timer() - start_time
    if seconds <= 60.0:
        return f"{seconds:.2f} secs"
    return f"{seconds / 60.0:.2f} min"



"""
Payload application
"""
import base64
import re
import traceback
from pathlib import Path
from typing import Type, List, Dict, Any, Union

from textual import work, on
from textual.app import App, CSSPathType, ComposeResult
from textual.containers import Center, Middle
from textual.driver import Driver
from textual.widgets import Header, Footer, ProgressBar, DataTable

from suricatalog.filter import WithPayloadFilter
from suricatalog.log import get_events_from_eve


class PayloadApp(App):
    """
    Base application for payload applications, shared logic
    No clipboard binding here as it makes no sense.
    """
    BINDINGS = [
        ("q", "quit_app", "Quit")
    ]
    ENABLE_COMMAND_PALETTE = False

    def __init__(
            self,
            driver_class: Type[Driver] | None = None,
            css_path: CSSPathType | None = None,
            watch_css: bool = False,
            eve: List[Path] = None,
            data_filter: WithPayloadFilter = WithPayloadFilter(),
            report_dir: Path = None,
    ):
        """
        Constructor
        :param driver_class:
        :param css_path:
        :param watch_css:
        :param eve:
        :param data_filter:
        """
        super().__init__(driver_class, css_path, watch_css)
        if not eve:
            raise ValueError("At least one Eve file is required")
        self.eve = eve
        if not data_filter:
            raise ValueError("Filter is required")
        self.data_filter = data_filter
        if not report_dir:
            raise ValueError("Destination report is missing")
        self.report_dir = report_dir
        self.loaded = 0

    @staticmethod
    def get_key_from_map(map1: Dict[str, Any], keys: List[str]):
        """
        Return the first matching key from a map
        :param map1:
        :param keys:
        :return: Nothing ig none of the keys are in the map
        """
        val = ""
        for key in keys:
            if key in map1:
                val = map1[key]
                break
        return val

    @staticmethod
    def convert_to_filename(orig: Union[str, None]) -> str:
        """
        Make sure string is safe with filesystem
        :param orig:
        :return:
        """
        if orig is None:
            return "no_signature"
        filename = re.sub(r'\W+', '', orig)
        filename = filename.replace(' ', '_')
        return filename

    @staticmethod
    def generate_filename(
            base_dir: Path,
            prefix: str = "payload_export",
            **payload_data: Dict[Any, Any],
    ) -> Path:
        """
        Generate a filename from payload components
        :param base_dir
        :param prefix:
        :param payload_data:
        :return:
        """
        data = payload_data.get('payload_data', payload_data)
        if not isinstance(data, dict):
            raise ValueError(f"payload_data must be a dict: {data}")
        timestamp = data.get("timestamp", None)
        if timestamp is None:
            raise ValueError(f"payload_data must have a timestamp: {data}")
        signature = data.get("signature", None)
        if signature is None:
            raise ValueError(f"payload_data must have a signature: {data}")
        signature = PayloadApp.convert_to_filename(signature)
        dest_port = data.get("dest_port", "")
        src_ip = data.get("src_ip", "")
        dest_ip = data.get("dest_ip", "")
        src_port = data.get("src_port", "")
        filename = f"{prefix}-{signature}-{timestamp}-{src_ip}:{src_port}-{dest_ip}:{dest_port}"
        return base_dir / filename

    @staticmethod
    async def extract_from_alert(
            alert: Dict[str, Any]
    ) -> Union[None, Dict[str, Any]]:
        """
        Extract alerts from event. Handle version changes between Suricata 6 and 7+.
        :param alert:
        :return:
        """
        if 'payload' not in alert:
            return None
        try:
            timestamp = alert['timestamp']
            dest_port = str(PayloadApp.get_key_from_map(alert, ['dest_port']))
            dest_ip = PayloadApp.get_key_from_map(alert, ['dest_ip'])
            src_ip = PayloadApp.get_key_from_map(alert, ['src_ip'])
            src_port = str(PayloadApp.get_key_from_map(alert, ['src_port']))
            payload = alert['payload']
            if 'signature' in alert:
                signature = alert['signature']
            else:
                signature = alert['alert']['signature']
            extracted = {
                "timestamp": timestamp,
                "dest_port": dest_port,
                "src_ip": src_ip,
                "dest_ip": dest_ip,
                "src_port": src_port,
                "payload": payload,
                "signature": signature
            }
            return extracted
        except KeyError as ke:
            raise KeyError(f"alert={alert}", ke) from ke

    def compose(self) -> ComposeResult:
        """
        Place TUI components
        :return:
        """
        yield Header()
        with Center():
            with Middle():
                yield ProgressBar(total=100, show_eta=False)
        yield Footer()

    @work(exclusive=False)
    async def on_mount(self) -> None:
        """
        Initialize TUI components
        :return:
        """
        await self.pump_events()
        if self.loaded > 0:
            self.notify(
                title="Finished extracting payloads from events",
                severity="information",
                timeout=5,
                message=f"Number of payload exported: {self.loaded}"
            )

    def action_quit_app(self) -> None:
        """
        Handle exit action
        :return:
        """
        self.exit("Exiting payload app now...")

    @staticmethod
    async def save_payload(payload: Dict[str, Any], payload_file: Path) -> int:
        """
        Save the extracted payload to disk
        :param payload:
        :param payload_file:
        :return:
        """
        with open(payload_file, "wb") as payload_fh:
            try:
                bin_payload = base64.b64decode(payload['payload'], altchars=None, validate=False)
                payload_fh.write(bin_payload)
                payload_fh.flush()
                return len(payload['payload'])
            except (TypeError, ValueError):
                payload_fh.write(payload['payload'])
                payload_fh.flush()
                return len(payload['payload'])

    @on(DataTable.HeaderSelected)
    def on_header_clicked(self, event: DataTable.HeaderSelected):
        """
        Handle clicks on table hear
        :param event:
        :return:
        """
        eve_files_tbl: DataTable = event.data_table
        eve_files_tbl.sort(event.column_key)

    @staticmethod
    def unique_id(extracted: Dict[Any, Any]) -> str:
        """
        Get the unique id based on extracted payload components
        :param extracted:
        :return:
        """
        if 'signature' not in extracted:
            raise ValueError("Missing signature")
        if 'timestamp' not in extracted:
            raise ValueError("Missing timestamp")
        uid = extracted['signature'] + str(extracted['timestamp'])
        return uid

    async def pump_events(self):
        """
        Get events from eve log and send them to the application log
        :param
        :return:
        """
        try:
            extract_ids = set([])
            # Need to count the events first. And don't want to store them in memory because the payload may be
            # huge...
            for alert_with_payload in get_events_from_eve(eve_files=self.eve, data_filter=self.data_filter):
                extracted = await PayloadApp.extract_from_alert(alert=alert_with_payload)
                uid = self.unique_id(extracted=extracted)
                extract_ids.add(uid)

            progress_bar = self.query_one(ProgressBar)
            self.loaded = 1
            for alert_with_payload in get_events_from_eve(eve_files=self.eve, data_filter=self.data_filter):
                extracted = await PayloadApp.extract_from_alert(alert=alert_with_payload)
                file_name = PayloadApp.generate_filename(
                    base_dir=self.report_dir,
                    payload_data=extracted
                )
                await self.save_payload(payload_file=file_name, payload=extracted)
                progress = (self.loaded / len(extract_ids)) * 100.0
                progress_bar.update(total=len(extract_ids), progress=progress)
                self.loaded += 1

        except ValueError as ve:
            if hasattr(ve, 'message'):
                reason = ve.message
            elif hasattr(ve, 'reason'):
                reason = f"{ve}"
            else:
                reason = f"{ve}"
            tb = traceback.extract_stack()
            self.notify(
                message=f"{reason}. {tb}",
                timeout=20,
                title="There was a problem extracting the payloads",
                severity="error"
            )
            raise ValueError("There was a problem extracting the payloads", ve) from ve



"""
Unit tests for clipboard data copy
"""
import unittest
from typing import List

from textual.widgets import Digits, RichLog, DataTable
from textual.app import App, ComposeResult
from textual.widgets import Header, Footer

from suricatalog.clipboard import copy_from_digits, copy_from_richlog, copy_from_table


class ClipboardRichLogDemoApp(App):
    """
    Test application to copy data from rich log
    """
    BINDINGS = [
        ("c", "copy", "Copy to clipboard")
    ]

    def __init__(
            self,
            log_update: str
    ):
        """
        Constructor
        :param log_update:
        """
        super().__init__()
        self.copied = None
        self.ln = -1
        self.update = log_update

    def compose(self) -> ComposeResult:
        """
        Compose items for the TUI
        :return:
        """
        yield Header()
        log = RichLog()
        log.write(self.update)
        log.write(self.update)
        yield log
        yield Footer()

    def action_copy(self):
        """
        Copy data to the clipboard
        :return:
        """
        log = self.query_one(RichLog)
        (self.copied, self.ln) = copy_from_richlog(log)


class ClipboardDataTableDemoApp(App):
    """
    Demo application to copy data from data table
    """
    BINDINGS = [
        ("c", "copy", "Copy to clipboard")
    ]

    def __init__(
            self,
            table_update: List[List[str]]
    ):
        """
        Constructor
        :param table_update:
        """
        super().__init__()
        self.copied = None
        self.ln = -1
        self.update = table_update

    def compose(self) -> ComposeResult:
        """
        Compose TUI elements
        :return:
        """
        yield Header()
        table = DataTable()
        table.add_columns(*self.update[0])
        table.add_rows(self.update[1:])
        yield table
        yield Footer()

    def action_copy(self):
        """
        Override action to copy data from data table
        :return:
        """
        table = self.query_one(DataTable)
        (self.copied, self.ln) = copy_from_table(table)


class ClipboardTestCase(unittest.IsolatedAsyncioTestCase):
    """
    Test cases for clipboard data copy
    """
    def test_copy_from_digits(self):
        """
        Copy digits
        :return:
        """
        digits = Digits()
        update = "Got a gazzillion bytes!"
        digits.update(update)
        copied, ln = copy_from_digits(digits)
        self.assertEqual(update, copied)
        self.assertEqual(23, ln)

    async def test_copy_from_richlog(self):
        """
        Copy rich log
        :return:
        """
        update = "Got 1973!"
        app = ClipboardRichLogDemoApp(log_update=update)
        self.assertIsNotNone(app)
        async with app.run_test() as pilot:
            await pilot.pause()
            await pilot.press("c")  # Copy to clipboard
            await pilot.pause()
        self.assertEqual('Got 1973!\nGot 1973!', app.copied)
        self.assertEqual(19, app.ln)

    async def test_copy_from_table(self):
        """
        Copy data from table
        :return:
        """
        update = [
            ['Login ID', 'Full Name'],
            ["1973", "Jose Vicente Nunez"],
            ["1973", "Jose Vicente Nunez"]
        ]
        app = ClipboardDataTableDemoApp(table_update=update)
        self.assertIsNotNone(app)
        async with app.run_test() as pilot:
            await pilot.pause()
            await pilot.press("c")  # Copy to clipboard
            await pilot.pause()
        self.assertEqual(86, app.ln)


if __name__ == '__main__':
    unittest.main()



"""
Unit tests for mini apps
"""
import unittest
from ipaddress import ip_address
from pathlib import Path

from textual.widgets import RichLog, DataTable, Digits

from suricatalog.canned import get_capture, get_one_shot_flow_table, get_host_data_use, get_agents
from suricatalog.filter import NXDomainFilter, WithPrintablePayloadFilter, TimestampFilter
from suricatalog.scripts.eve_json import ALWAYS_TRUE
from suricatalog.time import DEFAULT_TIMESTAMP_10Y_AGO

BASEDIR = Path(__file__).parent
EVE_FILE = BASEDIR.joinpath("eve.json")
EVE_UDP_FILE = BASEDIR.joinpath("eve_udp_flow.json")
OLD_DATE = DEFAULT_TIMESTAMP_10Y_AGO


class MiniAppsTestCase(unittest.IsolatedAsyncioTestCase):
    """
    Concrete implementation of unit test for mini apps
    """
    async def test_get_capture(self):
        """
        Simulate user iteration with mini apps
        :return:
        """
        app = get_capture(
            eve=[EVE_FILE],
            data_filter=NXDomainFilter(),
            title="SuricataLog DNS records with NXDOMAIN"
        )
        self.assertIsNotNone(app)
        async with app.run_test() as pilot:
            log = app.screen.query(RichLog).first()
            self.assertIsNotNone(log)
            self.assertEqual("        'rrtype': 'A',", log.lines[99].text)
            await pilot.pause()
            await pilot.press("q")

            app = get_capture(
                eve=[EVE_FILE],
                data_filter=WithPrintablePayloadFilter(),
                title="SuricataLog Inspect Alert Data (payload)"
            )
            self.assertIsNotNone(app)
        async with app.run_test() as pilot:
            log = app.screen.query(RichLog).first()
            self.assertIsNotNone(log)
            self.assertRegex("{", log.lines[0].text)
            await pilot.pause()
            await pilot.press("q")

    async def test_get_one_shot_flow_table(self):
        """
        Test one shot table app
        :return:
        """
        app = get_one_shot_flow_table(
            eve=[EVE_FILE],
            data_filter=ALWAYS_TRUE
        )
        self.assertIsNotNone(app)
        async with app.run_test() as pilot:
            table = app.query(DataTable).first()
            self.assertIsNotNone(table)
            coordinate = table.cursor_coordinate
            self.assertTrue(table.is_valid_coordinate(coordinate))
            data = table.get_cell_at(coordinate)
            self.assertEqual("TCP", data)
            await pilot.pause()
            await pilot.press("q")

    async def test_get_host_data_use(self):
        """
        Test get host data app
        :return:
        """
        ts = TimestampFilter()
        ts.timestamp = DEFAULT_TIMESTAMP_10Y_AGO
        app = get_host_data_use(
            eve_files=[EVE_UDP_FILE],
            data_filter=ts,
            ip_address=ip_address('224.0.0.251')
        )
        self.assertIsNotNone(app)
        async with app.run_test() as pilot:
            netflow = app.query(Digits).first()
            self.assertIsNotNone(netflow)
            self.assertEqual('0 bytes', netflow.value)
            await pilot.pause()
            await pilot.press("q")

    async def test_get_agents(self):
        """
        Test get agents app
        :return:
        """
        ts = TimestampFilter()
        ts.timestamp = DEFAULT_TIMESTAMP_10Y_AGO
        app = get_agents(
            eve_files=[EVE_FILE],
            data_filter=ts
        )
        self.assertIsNotNone(app)
        async with app.run_test() as pilot:
            log = app.screen.query(RichLog).first()
            self.assertIsNotNone(log)
            self.assertRegex("{", log.lines[0].text)
            await pilot.pause()
            await pilot.press("q")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
This script is inspired by the examples provided on
[15.1.3. Eve JSON ‘jq’ Examples](https://suricata.readthedocs.io/en/suricata-6.0.0/output/eve/eve-json-examplesjq.html)

A few things:
* The output uses colorized/ scrollable JSON
* You can filter by timestamp

"""

import argparse
import sys
from pathlib import Path
from ipaddress import ip_address

from suricatalog.filter import NXDomainFilter, WithPrintablePayloadFilter, AlwaysTrueFilter, TimestampFilter
from suricatalog.log import DEFAULT_EVE
from suricatalog.time import DEFAULT_TIMESTAMP_10Y_AGO, parse_timestamp
from suricatalog.canned import get_one_shot_flow_table, get_host_data_use, get_agents, get_capture

ALWAYS_TRUE = AlwaysTrueFilter()


def main():
    """
    CLI entry point
    :return:
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "--timestamp",
        type=parse_timestamp,
        default=DEFAULT_TIMESTAMP_10Y_AGO,
        help=f"Minimum timestamp in the past to use when filtering events ({DEFAULT_TIMESTAMP_10Y_AGO})"
    )
    exclusive_flags = parser.add_mutually_exclusive_group()
    exclusive_flags.add_argument(
        "--nxdomain",
        action='store_true',
        default=False,
        help="Show DNS records with NXDOMAIN"
    )
    exclusive_flags.add_argument(
        "--payload",
        action='store_true',
        default=False,
        help="Show alerts with a printable payload"
    )
    exclusive_flags.add_argument(
        "--flow",
        action='store_true',
        default=False,
        help="Aggregated flow report per protocol and destination port"
    )
    exclusive_flags.add_argument(
        "--netflow",
        type=ip_address,
        action='store',
        help="Get the netflow for a given IP address"
    )
    exclusive_flags.add_argument(
        "--useragent",
        action='store_true',
        default=False,
        help="Top user agent in HTTP traffic"
    )
    parser.add_argument(
        'eve_file',
        type=Path,
        nargs="+",
        help=f"Path to one or more {DEFAULT_EVE[0]} file to parse."
    )
    options = parser.parse_args()
    timestamp_filter = TimestampFilter()
    timestamp_filter.timestamp = options.timestamp
    try:
        if options.nxdomain:
            eve_app = get_capture(
                eve=options.eve_file,
                data_filter=NXDomainFilter(),
                title="SuricataLog DNS records with NXDOMAIN"
            )
        elif options.payload:
            eve_app = get_capture(
                eve=options.eve_file,
                data_filter=WithPrintablePayloadFilter(),
                title="SuricataLog Inspect Alert Data (payload)"
            )
        elif options.flow:
            eve_app = get_one_shot_flow_table(
                eve=options.eve_file,
                data_filter=ALWAYS_TRUE
            )
        elif options.netflow:
            eve_app = get_host_data_use(
                eve_files=options.eve_file,
                data_filter=timestamp_filter,
                ip_address=options.netflow.exploded
            )
        elif options.useragent:
            eve_app = get_agents(
                eve_files=options.eve_file,
                data_filter=timestamp_filter
            )
        else:
            parser.print_usage()
            sys.exit(0)
        eve_app.run()
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



"""
Unit test code for payload management
"""
import tempfile
import unittest
from pathlib import Path

from suricatalog.filter import WithPayloadFilter
from suricatalog.log import get_events_from_eve
from suricatalog.payload_app import PayloadApp

BASEDIR = Path(__file__).parent
EVE_FILES = [
    BASEDIR.joinpath("eve.json"),
    BASEDIR.joinpath("eve-2.json")
]


class PayloadTestCase(unittest.IsolatedAsyncioTestCase):

    """
    Payload unit tests
    """

    def test_get_payload_events(self):
        """
        Test get all event, unfiltered
        :return:
        """
        payload_filter = WithPayloadFilter()
        payload_events = list(get_events_from_eve(
            eve_files=EVE_FILES,
            data_filter=payload_filter
        ))
        self.assertIsNotNone(payload_events)
        self.assertEqual(131, len(payload_events))

    async def test_generate_filename(self):
        """
        Make sure generated test filename is correct
        :return:
        """
        payload_filter = WithPayloadFilter()
        payload_events = list(get_events_from_eve(
                eve_files=EVE_FILES,
                data_filter=payload_filter
        ))
        for payload_event in payload_events:
            self.assertIsNotNone(payload_event)
            extracted = await PayloadApp.extract_from_alert(alert=payload_event)
            filename = PayloadApp.generate_filename(
                base_dir=Path.home(),
                payload_data=extracted
            )
            self.assertIsNotNone(filename)
            # print(filename)

    async def test_extract_from_alert(self):
        """
        Check if payload filer can extract payload from alert
        :return:
        """
        payload_filter = WithPayloadFilter()
        payload_events = list(get_events_from_eve(
                eve_files=EVE_FILES,
                data_filter=payload_filter
        ))
        keys = {
            "timestamp",
            "dest_port",
            "src_ip",
            "dest_ip",
            "src_port",
            "payload",
            "signature"
        }
        for payload_event in payload_events:
            self.assertIsNotNone(payload_event)
            extracted = await PayloadApp.extract_from_alert(alert=payload_event)
            self.assertIsNotNone(extracted)
            self.assertEqual(keys, set(extracted.keys()))
            payload = extracted['payload']
            self.assertIsNotNone(payload)

    async def test_save_payload(self):
        """
        Test if payload can be saved
        :return:
        """
        with tempfile.NamedTemporaryFile(delete=True) as tmpdir:
            payload = {'payload': "FakeData"}
            saved = await PayloadApp.save_payload(
                payload_file=Path(tmpdir.name) / "",
                payload=payload
            )
            self.assertEqual(8, saved)


if __name__ == '__main__':
    unittest.main()



"""
Collection of mini apps, canned reports.
"""
import textwrap
from pathlib import Path
from typing import Type, List

import pyperclip
from textual import on
from textual.app import App, ComposeResult, CSSPathType
from textual.driver import Driver
from textual.widgets import Header, DataTable, Footer

from suricatalog.clipboard import copy_from_table
from suricatalog.filter import BaseFilter
from suricatalog.log import get_events_from_eve
from suricatalog.report import AggregatedFlowProtoReport


class FlowApp(App):
    """
    Flow traffic application
    """
    BINDINGS = [
        ("q", "quit_app", "Quit"),
        ("y,c", "copy_table", "Copy to clipboard")
    ]
    ENABLE_COMMAND_PALETTE = False

    def __init__(
            self,
            driver_class: Type[Driver] | None = None,
            css_path: CSSPathType | None = None,
            watch_css: bool = False,
            data_filter: BaseFilter = None,
            eve: List[Path] = None
    ):
        """
        Constructor
        :param driver_class:
        :param css_path:
        :param watch_css:
        :param data_filter:
        :param eve:
        """
        super().__init__(driver_class, css_path, watch_css)
        self.data_filter = data_filter
        self.eve = eve

    def action_quit_app(self) -> None:
        """
        Exit the application
        :return:
        """
        self.exit("Exiting Net-Flow now...")

    def action_copy_table(self) -> None:
        """
        Copy contents to the clipboard
        :return:
        """
        table_data = self.query_one(DataTable)
        try:
            _, ln = copy_from_table(table_data)
            self.notify(f"Copied {ln} characters!", title="Copied selection")
        except pyperclip.PyperclipException as exc:
            # Show a toast popup if we fail to copy.
            self.notify(
                str(exc),
                title="Clipboard error",
                severity="error",
            )

    def compose(self) -> ComposeResult:
        """
        Place components of the app on screen
        :return:
        """
        yield Header()
        alerts_tbl = DataTable()
        alerts_tbl.show_header = True
        alerts_tbl.add_column("Protocol")
        alerts_tbl.add_column("Port")
        alerts_tbl.add_column("Count")
        alerts_tbl.zebra_stripes = True
        alerts_tbl.loading = True
        alerts_tbl.cursor_type = 'row'
        alerts_tbl.tooltip = textwrap.dedent("""
        Network flow details
        """)
        yield alerts_tbl
        yield Footer()

    async def on_mount(self) -> None:
        """
        Populate components of the app on screen with relevant data
        :return:
        """
        alerts_tbl = self.query_one(DataTable)
        alert_cnt = 0
        afr = AggregatedFlowProtoReport()
        for event in get_events_from_eve(
                eve_files=self.eve,
                data_filter=self.data_filter):
            if not self.data_filter.accept(event):
                continue
            await afr.ingest_data(event)
        alerts_tbl.loading = False
        for (dest_ip_port, cnt) in afr.port_proto_count.items():
            alerts_tbl.add_row(
                dest_ip_port[0],
                str(dest_ip_port[1]),
                str(cnt)
            )
            alert_cnt += 1

    @on(DataTable.HeaderSelected)
    def on_header_clicked(self, event: DataTable.HeaderSelected):
        """
        Handle clicks on table hear
        :param event:
        :return:
        """
        alerts_tbl: DataTable = event.data_table
        alerts_tbl.sort(event.column_key)



"""
Discover code for table helpers
"""
from enum import Enum
from functools import partial
from typing import Any

from rich.style import Style
from textual.command import Provider, Hits, Hit, DiscoveryHit
from textual.screen import Screen
from textual.widgets import DataTable

from suricatalog.screens import DetailScreen


class TableColumns(Enum):
    """
    Columns for events
    """
    TIMESTAMP = 0
    SEVERITY = 1
    SIGNATURE = 2
    PROTOCOL = 3
    DESTINATION = 4
    SOURCE = 5
    PAYLOAD = 6


PROVIDER_COLS = [
    TableColumns.SIGNATURE,
    TableColumns.PROTOCOL,
    TableColumns.DESTINATION,
    TableColumns.SOURCE,
    TableColumns.PAYLOAD
]


class TableAlertProvider(Provider):
    """
    Event provider implementation
    """

    def __init__(self, screen: Screen[Any], match_style: Style | None = None):
        """
        Constructor
        :param screen:
        :param match_style:
        """
        super().__init__(screen, match_style)
        self.alerts_tbl = None

    async def startup(self) -> None:
        """
        initialization for the provider, when firts loaded
        :return:
        """
        self.alerts_tbl = self.app.query(DataTable).first()

    async def discover(self) -> DiscoveryHit:
        """
        Value discovery
        :return:
        """
        my_app = self.screen.app
        for row_key in self.alerts_tbl.rows:
            row = self.alerts_tbl.get_row(row_key)
            my_app.log.info(f"Searching {row_key}:{row}")
            for column in PROVIDER_COLS:
                searchable = row[column.value]
                matcher = self.matcher(searchable)
                runner_detail = DetailScreen(data=row)
                yield DiscoveryHit(
                    display=matcher.highlight(f"{searchable}"),
                    text=f"{column.name}: {searchable}",
                    command=partial(my_app.push_screen, runner_detail),
                    help=f"Select by {column.name}"
                )
            break  # Only care about the first result

    async def search(self, query: str) -> Hits:
        """
        Value search
        :param query:
        :return:
        """
        matcher = self.matcher(query)
        my_app = self.screen.app
        for row_key in self.alerts_tbl.rows:
            row = self.alerts_tbl.get_row(row_key)
            my_app.log.info(f"Searching {row_key}:{row}")
            for column in PROVIDER_COLS:
                searchable = row[column.value]
                score = matcher.match(searchable)
                if score > 0:
                    runner_detail = DetailScreen(data=row)
                    yield Hit(
                        score=score,
                        match_display=matcher.highlight(f"{searchable}"),
                        command=partial(my_app.push_screen, runner_detail),
                        text=f"{column.name}: {searchable}",
                        help=f"Select by {column.name}"
                    )



"""
Log file contents logic
"""
import os
import time
from pathlib import Path
import logging
from typing import Callable, Dict
import orjson
from orjson import JSONDecodeError

from suricatalog.filter import BaseFilter

DEFAULT_EVE = [Path("/var/log/suricata/eve.json")]
_fmt = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(lineno)d - %(message)s")
_sc = logging.StreamHandler()
_sc.setFormatter(_fmt)
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)
LOGGER.addHandler(_sc)


def get_events_from_eve(
        *,
        eve_files=None,
        data_filter: BaseFilter,
) -> Dict:
    """
    Get alerts from a JSON even file. Assumed each line is a valid
    JSON document, otherwise reader will crash
    :param eve_files:
    :param data_filter: Filter events based on several criteria
    :return:
    """
    if not isinstance(data_filter, BaseFilter):
        raise ValueError("Invalid 'data_filter' passed.")
    if eve_files is None:
        eve_files = DEFAULT_EVE
    for eve_file in eve_files:
        try:
            with open(eve_file, 'rt', encoding='utf-8') as eve:
                for line in eve:
                    try:
                        data = orjson.loads(line)
                        if data_filter.accept(data):
                            yield data
                    except JSONDecodeError:
                        LOGGER.exception("I cannot use data: '%s'. Ignoring it.", line)
                        continue  # Try to read the next record
        except (FileNotFoundError, FileExistsError, UnicodeDecodeError) as ve:
            LOGGER.exception("I cannot use file %s. Ignoring it.", eve_file)
            raise ValueError(f"I cannot use {eve_file}. Ignoring it.", ve) from ve


def tail_eve(
        *,
        eve_file=None,
        decorator: Callable = orjson.loads
):
    """
    Similar to UNIX ``tail -f eve.json``
    :param decorator:
    :param eve_file:
    :return:
    """
    if eve_file is None:
        eve_file = DEFAULT_EVE
    try:
        with open(eve_file, 'rt', encoding='utf-8') as eve:
            eve.seek(0, os.SEEK_END)
            while True:
                try:
                    line = eve.readline()
                    if not line:
                        time.sleep(0.1)
                        continue
                    data = decorator(line)
                    yield data
                except JSONDecodeError:
                    continue
    except (FileExistsError, UnicodeDecodeError) as ve:
        LOGGER.exception("I cannot use %s. Ignoring it.", eve_file)
        raise ValueError(f"I cannot use {eve_file}. Ignoring it.", ve) from ve



#!/usr/bin/env python
"""
Show Suricata alerts in different formats
Author: Jose Vicente Nunez (kodegeek.com@protonmail.com)
"""
import argparse
from pathlib import Path

from suricatalog.filter import BaseFilter, WithPrintablePayloadFilter
from suricatalog.alert_apps import TableAlertApp
from suricatalog.time import parse_timestamp, DEFAULT_TIMESTAMP_10Y_AGO
from suricatalog.log import DEFAULT_EVE


def main():
    """
    CLI entry point
    :return:
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "--timestamp",
        type=parse_timestamp,
        default=DEFAULT_TIMESTAMP_10Y_AGO,
        help=f"Minimum timestamp in the past to use when filtering events ({DEFAULT_TIMESTAMP_10Y_AGO})"
    )
    parser.add_argument(
        'eve_file',
        type=Path,
        nargs="+",
        help=f"Path to one or more {DEFAULT_EVE[0]} file to parse."
    )
    options = parser.parse_args()
    timestamp_filter: BaseFilter = WithPrintablePayloadFilter()
    timestamp_filter.timestamp = options.timestamp
    try:
        app = TableAlertApp()
        app.title = f"SuricataLog Alerts (filter='>={options.timestamp}') for {','.join([eve.name for eve in options.eve_file])}"
        app.set_filter(timestamp_filter)
        app.set_eve_files(options.eve_file)
        app.run()
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    """
    Entry level CLI
    """
    main()



"""
Application to extract payloads from Suricata eve log files.
:return:
"""
import argparse
from pathlib import Path

from suricatalog.filter import WithPayloadFilter
from suricatalog.log import DEFAULT_EVE
from suricatalog.payload_app import PayloadApp


def main():
    """
    CLI entry point
    :return:
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        '--report_dir',
        type=Path,
        required=True,
        help="Directory where the payloads must be stored. Needs to exist."
    )
    parser.add_argument(
        'eve_file',
        type=Path,
        nargs="+",
        help=f"Path to one or more {DEFAULT_EVE[0]} file to parse."
    )
    options = parser.parse_args()
    try:
        if not options.report_dir.exists():
            raise ValueError(f"Please continue {options.report_dir} before we can proceed to extract the payloads.")
        app = PayloadApp(
            eve=options.eve_file,
            data_filter=WithPayloadFilter(),
            report_dir=options.report_dir
        )
        app.title = f"SuricataLog Payload extractor. Working on {','.join(x.as_posix() for x in options.eve_file)}. Results to {options.report_dir.as_posix()}"
        app.run(inline=True)
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    """
    Entry level for CLI
    """
    main()



"""
Wrapper to execute eve CLI applications from a web browser.
Author: Jose Vicente Nunez (kodegeek.com@protonmail.com)

Examples:

   eve_server --application eve_json -- --flow ~/eve.json
   eve_server --application eve_json -- --nxdomain ~/eve.json
   eve_server --applications eve_log -- ~/eve.json

The -- is necessary to be able to process flags on all the other eve_* applications.

"""
import argparse

from textual_serve.server import Server

ALLOWED_APPS = [
    "eve_json",
    "eve_log"
]
PORT = 8000


def main():
    """
    Entry point for wrapper
    :return:
    """
    parser = argparse.ArgumentParser(
        description=__doc__,
        exit_on_error=False
    )
    parser.add_argument(
        "--application",
        action='store',
        choices=ALLOWED_APPS,
        default=ALLOWED_APPS[0],
        help=f"Allowed applications: {', '.join(ALLOWED_APPS)}"
    )
    parser.add_argument(
        "--port",
        action='store',
        default=PORT,
        required=False,
        help=f"Override port: {PORT}"
    )
    parser.add_argument(
        "--debug",
        action='store_true',
        default=False,
        help="Enable debug mode"
    )
    parser.add_argument(
        "eve_remainder",
        action='store',
        nargs=argparse.ZERO_OR_MORE,
        help="Arguments specific to suricata applications. Try passing --help"
    )

    options = parser.parse_args()
    cmd = f"{options.application} {' '.join(list(options.eve_remainder))}"
    server = Server(
        port=options.port,
        command=cmd,
        title="Suricatalog Server",
    )
    server.serve(options.debug)


if __name__ == "__main__":
    main()



"""
Canned report application
"""
from __future__ import annotations

from pathlib import Path
from typing import List
import locale

from textual.app import App

from suricatalog.flow_app import FlowApp
from suricatalog.hostdatause_app import HostDataUse
from suricatalog.topuser_app import TopUserApp
from suricatalog.oneshot_app import OneShotApp
from suricatalog.filter import BaseFilter

locale.setlocale(locale.LC_ALL, '')


def get_one_shot_flow_table(
        *,
        eve: List[Path],
        data_filter: BaseFilter
) -> App:
    """
    Helper to construct flow table
    :param eve:
    :param data_filter:
    :return:
    """
    flow_app = FlowApp(
        eve=eve,
        data_filter=data_filter
    )
    logs = ' '.join(map(str, eve))
    flow_app.title = f"SuricataLog FLOW protocol, logs={logs}"
    return flow_app


def get_host_data_use(
        eve_files: List[Path],
        data_filter: BaseFilter,
        ip_address: any
) -> App:
    """
    Helper to construct host data use
    :param eve_files:
    :param data_filter:
    :param ip_address:
    :return:
    """
    hdu = HostDataUse(
        eve=eve_files,
        data_filter=data_filter,
        ip_address=ip_address
    )
    hdu.title = f"SuricataLog Net-Flow for: {ip_address}"
    return hdu


def get_agents(
        eve_files: List[Path],
        data_filter: BaseFilter
) -> App:
    """
    Helper to construct common agents app
    :param eve_files:
    :param data_filter:
    :return:
    """
    top_user_app = TopUserApp(
        eve=eve_files,
        data_filter=data_filter
    )
    top_user_app.title = "SuricataLog User Agents"
    return top_user_app


def get_capture(
        *,
        eve: List[Path],
        data_filter: BaseFilter,
        title: str
) -> App:
    """
    Helper to construct capture app
    :param eve:
    :param data_filter:
    :param title:
    :return:
    """
    one_shot_app = OneShotApp(
        eve=eve,
        data_filter=data_filter
    )
    one_shot_app.title = f"{title}"
    return one_shot_app



"""
Unit test for alert applications
"""
import logging
import unittest
from pathlib import Path

from suricatalog.alert_apps import TableAlertApp, BaseAlertApp
from suricatalog.filter import OnlyAlertsFilter, WithPrintablePayloadFilter
from suricatalog.log import get_events_from_eve

BASEDIR = Path(__file__).parent
EVE_FILES = [
    BASEDIR.joinpath("eve-2.json"),
    BASEDIR.joinpath("eve.json")
]
_fmt = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(lineno)d - %(message)s")
_sc = logging.StreamHandler()
_sc.setFormatter(_fmt)
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)
LOGGER.addHandler(_sc)


class AlertAppsTestCase(unittest.IsolatedAsyncioTestCase):
    """
    Alert app unit test
    """
    async def test_extract_from_alert(self):
        """
        Text alert extraction
        :return:
        """
        for eve_file in EVE_FILES:
            events = get_events_from_eve(
                data_filter=WithPrintablePayloadFilter(),
                eve_files=[eve_file]
            )
            LOGGER.info("Processing %s", eve_file.as_posix())
            for event in events:
                brief_data = await BaseAlertApp.extract_from_alert(event)
                self.assertIsNotNone(brief_data)
                LOGGER.info("%s", brief_data)

    """
    Concrete unit test for alert app
    """

    async def test_eve_log(self):
        """
        Simulate user iteration with the alert app
        :return:
        """
        app = TableAlertApp()
        app.set_eve_files(EVE_FILES)
        app.set_filter(OnlyAlertsFilter())
        app.title = "Dummy title"
        self.assertIsNotNone(app)
        async with app.run_test() as pilot:
            await pilot.pause()
            # Quit the app by pressing q
            await pilot.press("q")


if __name__ == '__main__':
    unittest.main()
